from abc import ABC, abstractmethod


class AbstractCallbackDataBuilder(ABC):

    @abstractmethod
    def get_string(self, key, value=None) -> str:
        pass

    @abstractmethod
    def get_data(self, string: str):
        pass

    def get_key(self, string: str):
        key, _ = self.get_data(string)

        return key

    def get_value(self, string: str):
        _, value = self.get_data(string)

        return value



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type


@dataclass
class ForumTopicEdited(Type):
    name: Optional[str] = None
    icon_custom_emoji_id: Optional[str] = None



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type
from telebox.bot.types.types.chat_administrator_rights import ChatAdministratorRights


@dataclass
class KeyboardButtonRequestChat(Type):
    request_id: int
    chat_is_channel: bool
    chat_is_forum: Optional[bool] = None
    chat_has_username: Optional[bool] = None
    chat_is_created: Optional[bool] = None
    user_administrator_rights: Optional[ChatAdministratorRights] = None
    bot_administrator_rights: Optional[ChatAdministratorRights] = None
    bot_is_member: Optional[bool] = None



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type
from telebox.bot.consts import inline_query_result_types
from telebox.bot.types.types.inline_keyboard_markup import InlineKeyboardMarkup
from telebox.bot.types.types.input_message_content import InputMessageContent


@dataclass
class InlineQueryResultVenue(Type):
    id: str
    latitude: float
    longitude: float
    title: str
    address: str
    foursquare_id: Optional[str] = None
    foursquare_type: Optional[str] = None
    google_place_id: Optional[str] = None
    google_place_type: Optional[str] = None
    reply_markup: Optional[InlineKeyboardMarkup] = None
    input_message_content: Optional[InputMessageContent] = None
    thumbnail_url: Optional[str] = None
    thumbnail_width: Optional[int] = None
    thumbnail_height: Optional[int] = None
    type: str = inline_query_result_types.VENUE



from dataclasses import dataclass, field
from typing import Optional, BinaryIO
import os.path

from telebox.bot.types.type import Type


@dataclass
class InputFile(Type):
    file: BinaryIO = field(repr=False)
    name: Optional[str] = None

    def __post_init__(self):
        if (self.name is None) and hasattr(self.file, "name"):
            self.name = os.path.basename(self.file.name)



from dataclasses import dataclass
from typing import Union, Optional

from telebox.bot.types.type import Type
from telebox.bot.types.types.input_file import InputFile
from telebox.bot.types.types.mask_position import MaskPosition


@dataclass
class InputSticker(Type):
    sticker: Union[InputFile, str]
    emoji_list: list[str]
    mask_position: Optional[MaskPosition] = None
    keywords: Optional[list[str]] = None



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type


@dataclass
class KeyboardButtonRequestUser(Type):
    request_id: int
    user_is_bot: Optional[bool] = None
    user_is_premium: Optional[bool] = None



from dataclasses import dataclass
from typing import Optional

from telebox.bot.types.type import Type
from telebox.bot.types.types.user import User
from telebox.bot.types.types.order_info import OrderInfo


@dataclass
class PreCheckoutQuery(Type):
    id: str
    from_: User
    currency: str
    total_amount: int
    invoice_payload: str
    shipping_option_id: Optional[str] = None
    order_info: Optional[OrderInfo] = None

    @property
    def user_id(self) -> int:
        return self.from_.id
